<?php

namespace Drupal\custom_commerce\Plugin\Commerce\PromotionOffer;

use Drupal\commerce_order\Adjustment;
use Drupal\commerce_price\Price;
use Drupal\commerce_promotion\Entity\PromotionInterface;
use Drupal\commerce_promotion\Plugin\Commerce\PromotionOffer\OrderItemPercentageOff;
use Drupal\commerce_promotion\Plugin\Commerce\PromotionOffer\OrderPromotionOfferBase;
use Drupal\Core\Entity\EntityInterface;
use Drupal\Core\Form\FormStateInterface;

/**
 * @CommercePromotionOffer(
 *   id = "custom_commerce_custom_promotion",
 *   label = @Translation("Buy A and B get 50% on A+B"),
 *   entity_type = "commerce_order",
 * )
 */
class CustomPromotion extends OrderPromotionOfferBase {

    public function buildConfigurationForm(array $form, FormStateInterface $form_state) {
      $form = parent::buildConfigurationForm($form, $form_state);

      $default_first_item = \Drupal::entityTypeManager()->getStorage('commerce_product')->load($this->configuration['first_item']);
      $default_second_item = \Drupal::entityTypeManager()->getStorage('commerce_product')->load($this->configuration['second_item']);

      $form['first_item'] = [
        '#type' => 'entity_autocomplete',
        '#target_type' => 'commerce_product',
        '#title' => 'Выберите первый продукт',
        '#default_value' => !empty($default_first_item) ? $default_first_item : '',
      ];

      $form['second_item'] = [
        '#type' => 'entity_autocomplete',
        '#target_type' => 'commerce_product',
        '#title' => 'Выберите второй продукт',
        '#default_value' => !empty($default_second_item) ? $default_second_item : '',
      ];

      return $form;
    }

  public function submitConfigurationForm(array &$form, FormStateInterface $form_state) {
    if (!$form_state->getErrors()) {
      $values = $form_state->getValue($form['#parents']);
      $this->configuration['first_item'] = $values['first_item'];
      $this->configuration['second_item'] = $values['second_item'];
    }
  }

  public function apply(EntityInterface $entity, PromotionInterface $promotion) {
    $this->assertEntity($entity);
    /** @var \Drupal\commerce_order\Entity\OrderItemInterface $order_item */
    $order = $entity;
    $first_item = $this->configuration['first_item'];
    $second_item = $this->configuration['second_item'];

    $items = $order->getItems();
    foreach ($items as $item) {
      $variation = $item->getPurchasedEntity();
      $id_product_item = $variation->getProductId();
      if ($id_product_item == $first_item) {
        $first_item_order = $item;
        $first_unit_price = $item->getUnitPrice()->toArray();
      }
      if ($id_product_item == $second_item) {
        $second_item_order = $item;
        $second_unit_price = $item->getUnitPrice()->toArray();
      }
    }
    if (!empty($first_item_order) and !empty($second_item_order)) {
      if ($first_item_order->getQuantity() >= $second_item_order->getQuantity()) {
        $quantity = $second_item_order->getQuantity();
      } else {
        $quantity = $first_item_order->getQuantity();
      }

      $discount_first_item = $first_unit_price['number'] * $quantity;
      $discount_second_item = $second_unit_price['number'] * $quantity;

      $price = New Price($discount_first_item + $discount_second_item, $first_unit_price['currency_code']);

      $order->addAdjustment(new Adjustment([
        'type' => 'promotion',
        'label' => 'custom discount',
        'amount' => $price->multiply(-0.5),
        'source_id' => $promotion->id(),
      ]));
    }
  }
}
